/*===============================================================================
Title: deviceWISE Sample for Renesas
Description: deviceWISE Sample for Renesas RL78/G14 using Wifi

Copyright (c)2014, 2015, 2016 ILS Technology / Telit.

http://www.devicewise.com
http://www.help.devicewise.com

Legal Disclaimer:

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions
  are met:

    Neither the name of ILS Technology nor Telit nor the names of its
    contributors may be used to endorse or promote products derived
    from this software without specific prior written permission.

  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
  A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

===================================================================================*/

#include <stdint.h>
#include <stdlib.h>
#include <stdbool.h>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "RDKRL78_spi.h"
#include "r_cg_timer.h"
#include <sensors/LightSensor.h>
#include <sensors/Temperature.h>
#include <sensors/Potentiometer.h>
#include "led.h"

#include <init/hwsetup.h>
#include <GainSpan_SPI.h>

#include "dwOpenClient\dwOpenClient.h"

//Wi-Fi Network - SSID / Password
#define WIFI_SSID          "MyNetworkSSID"
#define WIFI_PW            "MyNetworkPW"

//Cloud Credentials 
#define DWOPEN_APPTOKEN    "bf2YxzRd44WxoDE8"  //Application Token
#define DWOPEN_SERVER      "api.devicewise.com"
#define DWOPEN_PORT        1883   


dwOpenClient dwClient(DWOPEN_SERVER, DWOPEN_PORT, callback);   

//ILST Headquarters - GPS Detail...
#define ILST_Latitude      "26.394919"
#define ILST_Longitude     "-80.112808"
#define ILST_Altitude      "3.0"
#define ILST_Speed         "0.0"
#define ILST_Heading       "0.0"     

#define GREEN_LED 1
#define RED_LED 2


//!!! 
//!!! COMMENT OUT ONE OR MORE SENSORS BELOW TO
//!!! MINIMIZE PROGRAM SIZE
//!!! 

#define USE_LIGHT_SENSOR
#define USE_TEMP_SENSOR
#define USE_POTENTIOMETER_SENSOR

char dwThingKey[40] = "RenesasRL78Thing"; 

extern char MRBuffer[];

//Renesas Specific Options and Security Descriptor 

#pragma location = "OPTBYTE"
__root const uint8_t opbyte0 = 0xEFU;
#pragma location = "OPTBYTE"
__root const uint8_t opbyte1 = 0xFFU;
#pragma location = "OPTBYTE"
__root const uint8_t opbyte2 = 0xF8U;
#pragma location = "OPTBYTE"
__root const uint8_t opbyte3 = 0x04U;
#pragma location = "SECUID"
__root const uint8_t secuid[10] = 
    {0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U, 0x00U};

void setupWiFi( char *SSID, char *Password)
{
        /* Give the unit a little time to start up */
        MSTimerDelay(1000);

        /* Clear out the buffers */
        AtLibGs_FlushRxBuffer();

        /* Send command to check */
        do {
            AtLibGs_FlushIncomingMessage();            
            MSTimerDelay(1000);    
        } while (ATLIBGS_MSG_ID_OK != AtLibGs_Check());

        /* Send command to DISABLE echo */
        do {
            MSTimerDelay(1000);            
        } while (ATLIBGS_MSG_ID_OK != AtLibGs_SetEcho(ATLIBGS_DISABLE) );
        
         
        //Connect to the WiFi
        do
        {   
            /* Try to disassociate if not already associated */
            AtLibGs_DisAssoc();
            while (1) 
            {  
                ATLIBGS_MSG_ID_E rxMsgId = AtLibGs_Mode(ATLIBGS_STATIONMODE_INFRASTRUCTURE);
                if (rxMsgId != ATLIBGS_MSG_ID_OK) 
                {
                    MSTimerDelay(2000);
                    continue;
                }        
                // Enable DHCP
                rxMsgId = AtLibGs_DHCPSet(1);
                if (rxMsgId != ATLIBGS_MSG_ID_OK) 
                {
                    MSTimerDelay(2000); 
                    continue;
                }
                break;
           }
              
           AtLibGs_CalcNStorePSK(SSID, Password);
           AtLibGs_Assoc(HOST_APP_AP_SSID, NULL, HOST_APP_AP_CHANNEL);                                
           
        } while( strstr(MRBuffer, "ERROR") );  
        
    return;
}

int  main(void)
{

    HardwareSetup(); 
    
    SPI2_Init();

    led_init();
    MSTimerInit();
    
    SPI_Init(GAINSPAN_SPI_RATE);  
   /* Setup LCD SPI channel for Chip Select P10, active low, active per byte  */
    SPI_ChannelSetup(GAINSPAN_SPI_CHANNEL, false, true);
    GainSpan_SPI_Start();
        
    // Set SAU0 enable and UART0/UART1 baud rate
    RL78G14RDK_UART_Start(GAINSPAN_CONSOLE_BAUD, GAINSPAN_CONSOLE_BAUD);
    PM15 &= ~(1 << 2);       //EInk hand
    P15 &= ~(1 << 2);
 
    MSTimerDelay(2000);    

    R_TAU0_Create();

#ifdef USE_LIGHT_SENSOR    
    LightSensor_Init();
#endif    
    
#ifdef USE_TEMP_SENSOR    
    Temperature_Init();
#endif
    
#ifdef USE_POTENTIOMETER_SENSOR    
    Potentiometer_Init();
#endif
    
    MSTimerDelay(2000);   

    printf("\n**** Renesas TL78 Demo Starting...\n");      
    
    setupWiFi( WIFI_SSID, WIFI_PW);
    
    led_all_off();  
    while(1) 
    {
            if (!dwClient.connected()) 
            {                    
                    bool connRC = dwClient.connect(dwThingKey, dwThingKey, DWOPEN_APPTOKEN );
                    
                    if ( !connRC )
                    {                            
                        dwClient.subscribe(MQTT_REPLY);     
                        AtLibGs_ReceiveDataHandle( (uint32_t) 5000);  

                        dwClient.subscribe(MQTT_NOTIFY);
                        AtLibGs_ReceiveDataHandle( (uint32_t) 5000);                           
                        
                        int rc = dwClient.dwLogPublish( (char*) dwThingKey, (char*) "Renesas Demo Program Started...");          
                               
                        rc = dwClient.dwLocationPublish( dwThingKey, ILST_Latitude, ILST_Longitude, ILST_Altitude, ILST_Speed, ILST_Heading );                 
        
                        rc = dwClient.dwAttributePublish( dwThingKey, (char *) "ID", (char *) "This is my ID" );                          
                    }
                                                                                         
            }            
                   
            
            if (dwClient.connected())
            {      
                    int16_t iValue;
                    char szData[100];                                 

#ifdef USE_TEMP_SENSOR                    
                    // Get our temp
                    float fValue;                    
                    iValue = Temperature_Get();
                    fValue = (((float)iValue) / 128.0) - 2.5;
                    fValue = (fValue * 1.8) + 32.0;
                    sprintf( szData,"%f", fValue );
                    
                    dwClient.dwPropertyPublish( dwThingKey, "Temperature", szData );        
#endif
                    
#ifdef USE_LIGHT_SENSOR                    
                    // Get our light
                    iValue = LightSensor_Get();
                    sprintf( szData,"%d", iValue );
                    
                    dwClient.dwPropertyPublish( dwThingKey, "LightSensor", szData );         
#endif
             
#ifdef USE_POTENTIOMETER_SENSOR                     
                    // Get our potentiometer value
                    iValue = Potentiometer_Get();
                    sprintf( szData,"%d", iValue );
                    
                    //printf("\n**Potentiometer\n");
                    dwClient.dwPropertyPublish( dwThingKey, "Potentiometer", szData );                           
#endif                    
                    //Delay a bit
                    MSTimerDelay(1000);
                    
                    //Wait and get any queued data
                    //AtLibGs_ReceiveDataHandle( (uint32_t) 55000);  
            }
    }
    
    AtLibGs_CloseAll();

    AtLibGs_ClearNodeAssociationFlag();

  

  return(0);
      
}

char needleTmp[40];
void ExtractKeyValue( char *haystack, char *Key, char *ResultValue )
{
  char *KeyPtr;
  char *KeyValue;
  char *KeyValueEnd;
  int   KeyValueLen;  
    	
    strcpy( needleTmp, "\"");
    strcat( needleTmp, Key);
    strcat( needleTmp, "\":\"");

    KeyPtr = strstr(haystack, needleTmp );
    	
    if( KeyPtr != NULL) 
    {	  
        KeyValue = KeyPtr + strlen(needleTmp);
        KeyValueEnd = KeyValue;
		
	while( *KeyValueEnd != '\"')
	{
	    KeyValueEnd++;
	}
		
        KeyValueLen = KeyValueEnd - KeyValue;
	strncpy( ResultValue, KeyValue, KeyValueLen );
	*(ResultValue + KeyValueLen ) = NULL;
    }
    else
    {
	strcpy( ResultValue, "" );
    }

}
  

// Sets the LED accordingly
void setLED( int LEDPin, char *LEDStatus )
{
    if ( strcmp(LEDStatus, "1") == NULL )
    {        
        printf("LED On!!!");          
       
        if ( LEDPin == GREEN_LED )
        {
            led_on(1);
            led_on(3);
            led_on(5);
            led_on(7);
            led_on(9);
            led_on(11);
        }
        else
        {
            led_on(0);
            led_on(2);
            led_on(4);
            led_on(6);
            led_on(8);
            led_on(10);          
        }
        
    }
    else
    {
        printf("LED Off!!!"); 
        
        if ( LEDPin == GREEN_LED )
        {
            led_off(1);
            led_off(3);
            led_off(5);
            led_off(7);
            led_off(9);
            led_off(11);
        }
        else
        {
            led_off(0);
            led_off(2);
            led_off(4);
            led_off(6);
            led_off(8);
            led_off(10);          
        }       
    }  
}


//TR50 JSON Sample Request/Response
#define MAILBOX_IN_PROGRESS_ACK  "{\"999-999\": { \"command\": \"mailbox.check\", \"params\": { \"autoComplete\": true, \"limit\": 0 }}}"
#define MAILBOX_REQ_RECEIVED     "{\"0000123\": { \"command\": \"mailbox.update\", \"params\": {\"id\": \"%s\", \"msg\": \"Executing...\"}}}"
#define MAILBOX_REQ_ACK          "{\"0000124\": { \"command\": \"mailbox.ack\", \"params\": {\"id\": \"%s\", \"errorCode\": \"0\", \"errorMessage\": \"OK\"}}}"


char payloadSave[1024];
char jsonText[1024];
char LEDStatus[10];   
char msgID[50];

void callback(char topic, uint8_t *payload, unsigned int length) 
{ 
    if ( topic == NOTIFY_MAILBOX_ACTIVITY_MSG )
    {            
        dwClient.publish("api", MAILBOX_IN_PROGRESS_ACK);
    }
    else if ( topic == REPLY_RESPONSE_MSG )
    {            
        if ((strstr((char *)payload, "999-999") != NULL)) 
        {                               
          strcpy( payloadSave, (char *) payload );
     
          ExtractKeyValue( (char *) payload, "id", msgID ); 
          
          sprintf( jsonText, MAILBOX_REQ_RECEIVED, msgID );  
          dwClient.publish("api", jsonText );
          
          //Set LED accordingly                 
          ExtractKeyValue( (char *) payloadSave, "greenled", LEDStatus );                
          if ( strlen(LEDStatus) ) setLED(GREEN_LED, LEDStatus);

          ExtractKeyValue( (char *) payloadSave, "redled", LEDStatus );                
          if ( strlen(LEDStatus) ) setLED(RED_LED, LEDStatus);
    
          //Send completion Ack back to the Cloud
          sprintf( jsonText, MAILBOX_REQ_ACK, msgID );                       
          dwClient.publish( "api", jsonText );  
        } 
          
    }
  
}