/*
  dwOpenClient.cpp - MQTT dwOpen Client Library

*/

#include "dwOpenClient.h"
#include "NVSettings.h"


NVSettings_t GNV_Setting;

extern char MRBuffer[];
extern uint8_t G_received[APP_MAX_RECEIVED_DATA + 1];
extern unsigned int G_receivedCount;
extern NVSettings_t G_nvsettings;
void App_PrepareIncomingData(void);

dwOpenClient::dwOpenClient() {
   this->_client = NULL;
   this->_isConnected = NULL;
}

dwOpenClient::dwOpenClient(uint8_t *ip, uint16_t port, void (*callback)(char,uint8_t*,unsigned int), Client& client) {
   this->_client = &client;
   this->_isConnected = NULL;   
   this->callback = callback;
   this->ip = ip;
   this->port = port;
   this->domain = NULL;
}

dwOpenClient::dwOpenClient(char* domain, uint16_t port, void (*callback)(char,uint8_t*,unsigned int)) {
   this->_client = NULL;
   this->_isConnected = NULL;  
   this->callback = callback;
   this->domain = domain;
   this->port = port;
}

bool dwOpenClient::connect(char *id) {
   return connect(id,NULL,NULL,0,0,0,0);
}

bool dwOpenClient::connect(char *id, char *user, char *pass) {
   return connect(id,user,pass,0,0,0,0);
}

bool dwOpenClient::connect(char *id, char* willTopic, uint8_t willQos, uint8_t willRetain, char* willMessage)
{
   return connect(id,NULL,NULL,willTopic,willQos,willRetain,willMessage);
}

bool dwOpenClient::connect(char *id, char *user, char *pass, char* willTopic, uint8_t willQos, uint8_t willRetain, char* willMessage) {
   char ipAddr[80];
   if (!connected()) 
   {     
       nextMsgId = 1;
       uint8_t d[9] = {0x00,0x06,'M','Q','I','s','d','p',MQTTPROTOCOLVERSION};
       // Leave room in the buffer for header and variable length field
       uint16_t length = 5;
       unsigned int j;
       for (j = 0;j<9;j++) {
          buffer[length++] = d[j];
       }

       uint8_t v;
       if (willTopic) {
          v = 0x06|(willQos<<3)|(willRetain<<5);
       } else {
          v = 0x02;
       }

       if(user != NULL) {
          v = v|0x80;

          if(pass != NULL) {
             v = v|(0x80>>1);
          }
       }

       buffer[length++] = v;

       buffer[length++] = ((MQTT_KEEPALIVE) >> 8);
       buffer[length++] = ((MQTT_KEEPALIVE) & 0xFF);
       length = writeString(id,buffer,length);
       if (willTopic) {
          length = writeString(willTopic,buffer,length);
          length = writeString(willMessage,buffer,length);
       }

       if(user != NULL) {
          length = writeString(user,buffer,length);
          if(pass != NULL) {
             length = writeString(pass,buffer,length);
          }
       }

      //Connect to the Portal
      AtLibGs_DNSLookup(this->domain, 3, 5);
      if ( AtLibGs_ParseDNSLookupResponse(ipAddr) )
      {
          //remove \r\n and a few other trailing characters
          ipAddr[strlen(ipAddr)-6] = '\0';
        
          AtLibGs_TCPClientStart( ipAddr, this->port, &cid);
          AtLibGs_SetNodeAssociationFlag();           
           
          //Send Connect Request
          write(MQTTCONNECT,buffer,length-5);         
          AtLibGs_ReceiveDataHandle( (uint32_t) 5000);
           
          //Check connect status...
          if ( strstr(MRBuffer, "CONNECT") )
          {
             _isConnected = 1;
             return(0);
          }
          else
          {
             _isConnected = 0;
             return(1);
          }
      }
      else
      {
          return(1);
      }
    }
   
   return(0);

}


uint16_t dwOpenClient::readPacket(uint16_t* lengthLength)
{
   uint16_t len = 0;
   buffer[len++] = readByte();
   uint8_t multiplier = 1;
   uint16_t length = 0;
   uint8_t digit = 0;

   do
   {
      digit = readByte();
      buffer[len++] = digit;
      length += (digit & 127) * multiplier;
      multiplier *= 128;
   } while ((digit & 128) != 0);

   *lengthLength = len-1;

   for(uint16_t i = 0; i < length; i++)
   {
      if (len < MQTT_MAX_PACKET_SIZE)
      {
         buffer[len++] = readByte();
      }
      else
      {
         readByte();
         len = 0; // This will cause the packet to be ignored.
      }
   }
   
   return len;
}

bool dwOpenClient::publish(char* topic, char* payload) {
   return publish(topic,(uint8_t*)payload,strlen(payload),false);
}

bool dwOpenClient::publish(char* topic, uint8_t* payload, unsigned int plength) {
   return publish(topic, payload, plength, false);
}

bool dwOpenClient::publish(char* topic, uint8_t* payload, unsigned int plength, bool retained) {
   if (connected()) {
      // Leave room in the buffer for header and variable length field
      uint16_t length = 5;
      length = writeString(topic,buffer,length);
      uint16_t i;
      for (i=0;i<plength;i++) {
         buffer[length++] = payload[i];
      }
      uint8_t header = MQTTPUBLISH;
      if (retained) {
         header |= 1;
      }
      
      printf("\nSending:%s\n", payload );
      
      bool wrtRC = write(header,buffer,length-5);
      int  retryCount = 0;
      do
      {
          AtLibGs_ReceiveDataHandle( (uint32_t) 5000); 
          
          retryCount++;
          if ( retryCount > 5 ) 
          {
              _isConnected = false;
              return(true);
          }
     
      } while( !G_receivedCount );
 
      //check if mailbox was received
      for ( int i = 0; i < APP_MAX_RECEIVED_DATA; i++ )
      {
          if ( G_received[i] == NOTIFY_MAILBOX_ACTIVITY_MSG )
          {         
              callback(NOTIFY_MAILBOX_ACTIVITY_MSG, &G_received[i+1], strlen( (char *) &G_received[i+1]) );              
              
              //re-queue this outstanding publish request that was interrupted by the mailbox message
              publish(topic, payload, plength, retained);           
            
              break;
          }
          else if ( G_received[i] == REPLY_RESPONSE_MSG )
          {
              if ( strstr( (char *)payload, MAILBOX_CHECK_MSG) )
              {              
                  callback(REPLY_RESPONSE_MSG, &G_received[i+1], strlen( (char *) &G_received[i+1]) );
              }
              else 
              {         
                  printf("** Resp : %s\n", &G_received[i+1] );
              }
              
              break;
          }
      }              

   }
   
   return false;
}

bool dwOpenClient::write(uint8_t header, uint8_t* buf, uint16_t length) {
   uint8_t lenBuf[4];
   uint8_t llen = 0;
   uint8_t digit;
   uint8_t pos = 0;
   uint16_t len = length;
   do {
      digit = len % 128;
      len = len / 128;
      if (len > 0) {
         digit |= 0x80;
      }
      lenBuf[pos++] = digit;
      llen++;
   } while(len>0);

   buf[4-llen] = header;
   for (int i=0;i<llen;i++) {
      buf[5-llen+i] = lenBuf[i];
   }
   
   memcpy( buffer, buf+(4-llen), length+1+llen);
   

   App_PrepareIncomingData();
   
   ATLIBGS_MSG_ID_E xrc = AtLibGs_SendTCPData( cid, (const void *) buffer, (uint16_t) length+1+llen);
 
   return (xrc);
}

bool dwOpenClient::subscribe(char* topic) {
   if (connected()) {
      // Leave room in the buffer for header and variable length field
      uint16_t length = 5;
      nextMsgId++;
      if (nextMsgId == 0) {
         nextMsgId = 1;
      }
      buffer[length++] = (nextMsgId >> 8);
      buffer[length++] = (nextMsgId & 0xFF);
      length = writeString(topic, buffer,length);
      buffer[length++] = 0; // Only do QoS 0 subs
      return write(MQTTSUBSCRIBE|MQTTQOS1,buffer,length-5);
       
   }
   return false;
}

bool dwOpenClient::unsubscribe(char* topic) {
   if (connected()) {
      uint16_t length = 5;
      nextMsgId++;
      if (nextMsgId == 0) {
         nextMsgId = 1;
      }
      buffer[length++] = (nextMsgId >> 8);
      buffer[length++] = (nextMsgId & 0xFF);
      length = writeString(topic, buffer,length);
      return write(MQTTUNSUBSCRIBE|MQTTQOS1,buffer,length-5);
   }
   return false;
}

void dwOpenClient::disconnect() {
   buffer[0] = MQTTDISCONNECT;
   buffer[1] = 0;
#if 0   
   _client->write(buffer,2);
   _client->stop();
   lastInActivity = lastOutActivity = millis();
#endif   
}

uint16_t dwOpenClient::writeString(char* string, uint8_t* buf, uint16_t pos) {
   char* idp = string;
   uint16_t i = 0;
   pos += 2;
   while (*idp) {
      buf[pos++] = *idp++;
      i++;
   }
   buf[pos-i-2] = (i >> 8);
   buf[pos-i-1] = (i & 0xFF);
   return pos;
}


bool dwOpenClient::connected() {
   bool rc;
   if (_isConnected == NULL ) {
      rc = false;
   } else {
      rc = 1;  
   }
   return rc;
}

// Generate JSON BindThing Request
void dwOpenClient::jsonCreateBindThing( char *thingKey, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"thing.bind\", \"params\": { \"key\": \"%s\" } } }", msgId, thingKey );

    return;
}

// Generate JSON UnbindThing Request
void dwOpenClient::jsonCreateUnbindThing( char *thingKey, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"thing.unbind\", \"params\": { \"key\": \"%s\" } } }", msgId, thingKey );

    return;
}

void dwOpenClient::jsonCreateLogPublish( char *thingKey, char *msgText, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"log.publish\", \"params\": {\"msg\": \"%s\", \"thingKey\": \"%s\" }}}", msgId, msgText, thingKey );

    return;
}

void dwOpenClient::jsonCreatePropertyPublish( char *thingKey, char *propKey, char *propValue, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"property.publish\", \"params\": {\"thingKey\": \"%s\",\"key\": \"%s\",\"value\": %s }}}", msgId, thingKey, propKey, propValue );

    return;
}

void dwOpenClient::jsonCreatePropertyCurrent( char *thingKey, char *propKey, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"property.current\", \"params\": {\"thingKey\": \"%s\",\"key\": \"%s\" }}}", msgId, thingKey, propKey );

    return;
}

void dwOpenClient::jsonCreateAlarmPublish( char *thingKey, char *alarmKey, int alarmState, char *alarmMsg, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"alarm.publish\", \"params\": {\"thingKey\": \"%s\", \"key\": \"%s\", \"msg\": \"%s\", \"state\": %d}}}", msgId, thingKey, alarmKey, alarmMsg, alarmState );

    return;
}

void dwOpenClient::jsonCreateLocationPublish( char *thingKey, char *locLatitude, char *locLongitude, char *locAltitude, char *locSpeed, char *locHeading, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"location.publish\", \"params\": {\"thingKey\": \"%s\", \"lat\": \"%s\", \"lng\": %s, \"altitude\": %s, \"speed\": %s, \"alti\": %s}}}", msgId, thingKey, locLatitude, locLongitude, locAltitude, locSpeed, locHeading );

    return;
}

void dwOpenClient::jsonCreateSetAttribute( char *thingKey, char *attribKey, char *attribValue, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"thing.attr.set\", \"params\": {\"thingKey\": \"%s\", \"key\": \"%s\", \"value\": \"%s\"}}}", msgId, thingKey, attribKey, attribValue );

    return;
}

void dwOpenClient::jsonCreateGetAttribute( char *thingKey, char *attribKey, char *jsonText )
{
    // Create JSON Command Request...
    msgId++;
    sprintf(jsonText, "{ \"%d\": { \"command\": \"thing.attr.get\", \"params\": {\"thingKey\": \"%s\", \"key\": \"%s\"}}}", msgId, thingKey, attribKey );

    return;
}

int dwOpenClient::dwLogPublish( char *thingKey, char *szLogMsg )
{
    // Publish Log Message...
    jsonCreateLogPublish( thingKey, szLogMsg, szBuffer );
    rc = publish( (char*) "api", szBuffer );

    MSTimerDelay(1000);

    return rc;
}

int dwOpenClient::dwLocationPublish( char *thingKey, char *szLat, char *szLong, char *szAlt, char *szSpeed, char *szHeading )
{
    // Publish Location Message...
    jsonCreateLocationPublish( thingKey, szLat, szLong, szAlt, szSpeed, szHeading, szBuffer );
    rc = publish( (char*) "api", szBuffer );

    MSTimerDelay(1000);

    return rc;
}

int dwOpenClient::dwPropertyPublish( char *thingKey, char *szKey, char *szValue )
{
    // Publish Property Message...
    jsonCreatePropertyPublish( thingKey, szKey, szValue, szBuffer );
    rc = publish( (char*) "api", szBuffer );

    MSTimerDelay(1000);

    return rc;
}

int dwOpenClient::dwAttributePublish( char *thingKey, char *szKey, char *szValue )
{
    // Publish Attribute Message...
    jsonCreateSetAttribute( thingKey, szKey, szValue, szBuffer );
	
    rc = publish( (char*) "api", szBuffer );

    MSTimerDelay(1000);

    return rc;
}

int dwOpenClient::dwAlarmPublish( char *thingKey, char *szKey, int iState, char *szMessage )
{
    // Publish Property Message...
    jsonCreateAlarmPublish( thingKey, szKey, iState, szMessage, szBuffer );
    rc = publish( (char*) "api", szBuffer );

    MSTimerDelay(1000);

    return rc;
}




